@@ -11,8 +11,8 @@ vd.beforeExecHooks = [] # func(sheet, cmd, args, keystrokes) called before the e
 
 class LazyChainMap:
     'provides a lazy mapping to obj attributes.  useful when some attributes are expensive properties.'
-    def __init__(self, *objs):
-        self.locals = {}
+    def __init__(self, *objs, locals=None):
+        self.locals = {} if locals is None else locals
         self.objs = {} # [k] -> obj
         for obj in objs:
             for k in dir(obj):
@@ -92,7 +92,7 @@ class BaseSheet(DrawablePane):
     def __init__(self, *names, **kwargs):
         self._name = None   # initial cache value necessary for self.options
         self.names = names
-        self.name = self.options.name_joiner.join(str(x) for x in self.names)
+        self.name = self.options.name_joiner.join(str(x) for x in self.names if x)
         self.source = None
         self.rows = UNLOADED      # list of opaque objects
         self._scr = mock.MagicMock(__bool__=mock.Mock(return_value=False))  # disable curses in batch mode
@@ -102,6 +102,8 @@ class BaseSheet(DrawablePane):
 
         super().__init__(**kwargs)
 
+        self._sidebar = ''
+
     def setModified(self):
         if not self.hasBeenModified:
             vd.addUndo(setattr, self, 'hasBeenModified', self.hasBeenModified)
@@ -247,6 +249,21 @@ class BaseSheet(DrawablePane):
         'Evaluate Python expression *expr* in the context of *kwargs* (may vary by sheet type).'
         return eval(expr, vd.getGlobals(), None)
 
+    @property
+    def sidebar(self):
+        'Default implementation just returns set value.  Overrideable.'
+        return self._sidebar
+
+    @sidebar.setter
+    def sidebar(self, v):
+        'Default implementation just sets value.  Overrideable.'
+        self._sidebar = v
+
+    @property
+    def sidebar_title(self):
+        'Default implementation returns fixed value.  Overrideable.'
+        return 'sidebar'
+
 
 @VisiData.api
 def redraw(vd):
