@@ -1,16 +1,19 @@
+import math
+import random
 
 from collections import defaultdict, Counter
 from visidata import *
+from visidata.bezier import bezier
 
 # see www/design/graphics.md
 
-option('show_graph_labels', True, 'show axes and legend on graph')
-theme('plot_colors', 'green red yellow cyan magenta white 38 136 168', 'list of distinct colors to use for plotting distinct objects')
-theme('disp_canvas_charset', ''.join(chr(0x2800+i) for i in range(256)), 'charset to render 2x4 blocks on canvas')
-theme('disp_pixel_random', False, 'randomly choose attr from set of pixels instead of most common')
-option('zoom_incr', 2.0, 'amount to multiply current zoomlevel when zooming')
-theme('color_graph_hidden', '238 blue', 'color of legend for hidden attribute')
-theme('color_graph_selected', 'bold', 'color of selected graph points')
+vd.option('show_graph_labels', True, 'show axes and legend on graph')
+vd.option('plot_colors', 'green red yellow cyan magenta white 38 136 168', 'list of distinct colors to use for plotting distinct objects')
+vd.option('disp_canvas_charset', ''.join(chr(0x2800+i) for i in range(256)), 'charset to render 2x4 blocks on canvas')
+vd.option('disp_pixel_random', False, 'randomly choose attr from set of pixels instead of most common')
+vd.option('zoom_incr', 2.0, 'amount to multiply current zoomlevel when zooming')
+vd.option('color_graph_hidden', '238 blue', 'color of legend for hidden attribute')
+vd.option('color_graph_selected', 'bold', 'color of selected graph points')
 
 
 class Point:
@@ -189,7 +192,9 @@ class Plotter(BaseSheet):
     def getPixelAttrMost(self, x, y):
         'most common attr at this pixel.'
         r = self.pixels[y][x]
-        c = [(len(rows), attr, rows) for attr, rows in list(r.items()) if attr and attr not in self.hiddenAttrs]
+        if not r:
+            return 0
+        c = [(len(rows), attr, rows) for attr, rows in r.items() if attr and attr not in self.hiddenAttrs]
         if not c:
             return 0
         _, attr, rows = max(c)
@@ -341,11 +346,12 @@ class Canvas(Plotter):
     def plotColor(self, k):
         attr = self.plotAttrs.get(k, None)
         if attr is None:
-            if len(self.unusedAttrs) > 1:
+            if self.unusedAttrs:
                 attr = self.unusedAttrs.pop(0)
                 legend = ' '.join(str(x) for x in k)
             else:
-                attr = self.unusedAttrs[0]
+                lastlegend, attr = list(self.legends.items())[-1]
+                del self.legends[lastlegend]
                 legend = '[other]'
 
             self.legendwidth = max(self.legendwidth, len(legend))
@@ -442,63 +448,8 @@ class Canvas(Plotter):
         x2, y2 = vertexes[1]
         x3, y3 = vertexes[2]
 
-        self.point(x1, y1, attr, row)
-        self._recursive_bezier(x1, y1, x2, y2, x3, y3, attr, row)
-        self.point(x3, y3, attr, row)
-
-    def _recursive_bezier(self, x1, y1, x2, y2, x3, y3, attr, row, level=0):
-        'from http://www.antigrain.com/research/adaptive_bezier/'
-        m_approximation_scale = 10.0
-        m_distance_tolerance = (0.5 / m_approximation_scale) ** 2
-        m_angle_tolerance = 1 * 2*math.pi/360  # 15 degrees in rads
-        curve_angle_tolerance_epsilon = 0.01
-        curve_recursion_limit = 32
-        curve_collinearity_epsilon = 1e-30
-
-        if level > curve_recursion_limit:
-            return
-
-        # Calculate all the mid-points of the line segments
-
-        x12   = (x1 + x2) / 2
-        y12   = (y1 + y2) / 2
-        x23   = (x2 + x3) / 2
-        y23   = (y2 + y3) / 2
-        x123  = (x12 + x23) / 2
-        y123  = (y12 + y23) / 2
-
-        dx = x3-x1
-        dy = y3-y1
-        d = abs(((x2 - x3) * dy - (y2 - y3) * dx))
-
-        if d > curve_collinearity_epsilon:
-            # Regular care
-            if d*d <= m_distance_tolerance * (dx*dx + dy*dy):
-                # If the curvature doesn't exceed the distance_tolerance value, we tend to finish subdivisions.
-                if m_angle_tolerance < curve_angle_tolerance_epsilon:
-                    self.point(x123, y123, attr, row)
-                    return
-
-                # Angle & Cusp Condition
-                da = abs(math.atan2(y3 - y2, x3 - x2) - math.atan2(y2 - y1, x2 - x1))
-                if da >= math.pi:
-                    da = 2*math.pi - da
-
-                if da < m_angle_tolerance:
-                    # Finally we can stop the recursion
-                    self.point(x123, y123, attr, row)
-                    return
-        else:
-            # Collinear case
-            dx = x123 - (x1 + x3) / 2
-            dy = y123 - (y1 + y3) / 2
-            if dx*dx + dy*dy <= m_distance_tolerance:
-                self.point(x123, y123, attr, row)
-                return
-
-        # Continue subdivision
-        self._recursive_bezier(x1, y1, x12, y12, x123, y123, attr, row, level + 1)
-        self._recursive_bezier(x123, y123, x23, y23, x3, y3, attr, row, level + 1)
+        for x, y in bezier(x1, y1, x2, y2, x3, y3):
+            self.point(x, y, attr, row)
 
     def label(self, x, y, text, attr=0, row=None):
         self.gridlabels.append((x, y, text, attr, row))
@@ -606,7 +557,7 @@ class Canvas(Plotter):
     def render(self, h, w):
         'resets plotter, cancels previous render threads, spawns a new render'
         self.needsRefresh = False
-        cancelThread(*(t for t in self.currentThreads if t.name == 'plotAll_async'))
+        vd.cancelThread(*(t for t in self.currentThreads if t.name == 'plotAll_async'))
         self.labels.clear()
         self.resetCanvasDimensions(h, w)
         self.render_async()
@@ -700,9 +651,8 @@ Canvas.addCommand('BUTTON1_RELEASED', 'end-cursor', 'setCursorSize(canvasMouse)'
 Canvas.addCommand('BUTTON3_PRESSED', 'start-move', 'sheet.anchorPoint = canvasMouse', 'mark grid point to move')
 Canvas.addCommand('BUTTON3_RELEASED', 'end-move', 'fixPoint(plotterMouse, anchorPoint)', 'mark canvas anchor point')
 
-Canvas.addCommand('BUTTON4_PRESSED', 'zoomin-mouse', 'tmp=canvasMouse; incrZoom(1.0/options.zoom_incr); fixPoint(plotterMouse, tmp)', 'zoom in with scroll wheel')
-Canvas.addCommand('REPORT_MOUSE_POSITION', 'zoomout-mouse', 'tmp=canvasMouse; incrZoom(options.zoom_incr); fixPoint(plotterMouse, tmp)', 'zoom out with scroll wheel')
-Canvas.bindkey('2097152', 'zoomout-mouse')
+Canvas.addCommand('ScrollwheelUp', 'zoomin-mouse', 'tmp=canvasMouse; incrZoom(1.0/options.zoom_incr); fixPoint(plotterMouse, tmp)', 'zoom in with scroll wheel')
+Canvas.addCommand('ScrollwheelDown', 'zoomout-mouse', 'tmp=canvasMouse; incrZoom(options.zoom_incr); fixPoint(plotterMouse, tmp)', 'zoom out with scroll wheel')
 
 Canvas.addCommand('s', 'select-cursor', 'source.select(list(rowsWithin(plotterCursorBox)))', 'select rows on source sheet contained within canvas cursor')
 Canvas.addCommand('t', 'stoggle-cursor', 'source.toggle(list(rowsWithin(plotterCursorBox)))', 'toggle selection of rows on source sheet contained within canvas cursor')
@@ -715,3 +665,12 @@ Canvas.addCommand('gt', 'stoggle-visible', 'source.toggle(list(rowsWithin(plotte
 Canvas.addCommand('gu', 'unselect-visible', 'source.unselect(list(rowsWithin(plotterVisibleBox)))', 'unselect rows on source sheet visible on screen')
 Canvas.addCommand('g'+ENTER, 'dive-visible', 'vs=copy(source); vs.rows=list(rowsWithin(plotterVisibleBox)); vd.push(vs)', 'open sheet of source rows visible on screen')
 Canvas.addCommand('gd', 'delete-visible', 'deleteSourceRows(rowsWithin(plotterVisibleBox))', 'delete rows on source sheet visible on screen')
+
+
+vd.addGlobals({
+    'Canvas': Canvas,
+    'Plotter': Plotter,
+    'BoundingBox': BoundingBox,
+    'Box': Box,
+    'Point': Point,
+})
