@@ -1,6 +1,6 @@
-import codecs
 import tarfile
 import zipfile
+from visidata.loaders import unzip_http
 
 from visidata import vd, VisiData, asyncthread, Sheet, Progress, Menu, options
 from visidata import ColumnAttr, Column, date, datetime, Path
@@ -20,14 +20,19 @@ VisiData.open_tbz2 = VisiData.open_tar
 @VisiData.api
 class ZipSheet(Sheet):
     'Wrapper for `zipfile` library.'
-    rowtype = 'files' # rowdef ZipInfo
+    rowtype = 'files' # rowdef [ZipInfo, zipfile.Path]
     columns = [
-        ColumnAttr('filename'),
-        ColumnAttr('file_size', type=int),
+        Column('directory',
+            getter=lambda col,row: str(row[1].parent) if str(row[1].parent) == '.' else str(row[1].parent) + '/'),
+        Column('filename', getter=lambda col,row: row[1].name + row[1].suffix),
+        Column('abspath', type=str, width=0, getter=lambda col,row: row[1]),
+        Column('ext', getter=lambda col,row: row[0].filename.endswith('/') and '/' or row[1].ext),
+        Column('size', getter=lambda col,row: row[0].file_size, type=int),
+        Column('compressed_size', type=int, getter=lambda col,row: row[0].compress_size),
         Column('date_time', type=date,
-               getter=lambda col, row: datetime.datetime(*row.date_time)),
-        ColumnAttr('compress_size', type=int)
+               getter=lambda col, row: datetime.datetime(*row[0].date_time)),
     ]
+    nKeys = 2
 
     def openZipFile(self, fp, *args, **kwargs):
         '''Use VisiData input to handle password-protected zip files.'''
@@ -39,24 +44,31 @@ class ZipSheet(Sheet):
                 return fp.open(*args, **kwargs, pwd=pwd.encode('utf-8'))
             vd.error(err)
 
-    def openRow(self, fi):
-            decodedfp = codecs.iterdecode(self.openZipFile(self.zfp, fi),
-                                          encoding=options.encoding,
-                                          errors=options.encoding_errors)
-            return vd.openSource(Path(fi.filename, fp=decodedfp, filesize=fi.file_size), filetype=options.filetype)
+    def openRow(self, row):
+            fi, zpath = row
+            fp = self.openZipFile(self.zfp, fi)
+            return vd.openSource(Path(fi.filename, fp=fp, filesize=fi.file_size), filetype=options.filetype)
 
     @asyncthread
     def extract(self, *rows, path=None):
-        self.zfp.extractall(members=[r.filename for r in rows], path=path)
+        for r, _ in Progress(rows):
+            self.zfp.extractall(members=[r.filename], path=path)
+            vd.status(f'extracted {r.filename}')
 
     @property
     def zfp(self):
-        return zipfile.ZipFile(str(self.source), 'r')
+        if not self._zfp:
+            if '://' in str(self.source):
+                unzip_http.warning = vd.warning
+                self._zfp = unzip_http.RemoteZipFile(str(self.source))
+            else:
+                self._zfp = zipfile.ZipFile(str(self.source), 'r')
+
+        return self._zfp
 
     def iterload(self):
-        with self.zfp as zf:
-            for zi in Progress(zf.infolist()):
-                yield zi
+        for zi in Progress(self.zfp.infolist()):
+            yield [zi, Path(zi.filename)]
 
 
 class TarSheet(Sheet):
@@ -74,10 +86,7 @@ class TarSheet(Sheet):
 
     def openRow(self, fi):
             tfp = tarfile.open(name=str(self.source))
-            decodedfp = codecs.iterdecode(tfp.extractfile(fi),
-                                          encoding=options.encoding,
-                                          errors=options.encoding_errors)
-            return vd.openSource(Path(fi.name, fp=decodedfp, filesize=fi.size))
+            return vd.openSource(Path(fi.name, fp=tfp.extractfile(fi), filesize=fi.size))
 
     def iterload(self):
         with tarfile.open(name=str(self.source)) as tf:
@@ -85,6 +94,8 @@ class TarSheet(Sheet):
                 yield ti
 
 
+ZipSheet.init('_zfp', lambda: None, copy=True)
+
 ZipSheet.addCommand('x', 'extract-file', 'extract(cursorRow)', 'extract current file to current directory')
 ZipSheet.addCommand('gx', 'extract-selected', 'extract(*onlySelectedRows)', 'extract selected files to current directory')
 ZipSheet.addCommand('zx', 'extract-file-to', 'extract(cursorRow, path=inputPath("extract to: "))', 'extract current file to given pathname')
