@@ -6,13 +6,19 @@ import threading
 import time
 from unittest import mock
 
-from visidata import vd, VisiData, colors, ESC, options, option
+from visidata import vd, VisiData, colors, ESC, options, clipbox
+
+__all__ = ['ReturnValue', 'run']
 
 vd.curses_timeout = 100 # curses timeout in ms
 vd.timeouts_before_idle = 10
+vd.min_draw_ms = 100  # draw_all at least this often, even if keystrokes are pending
+vd._lastDrawTime = 0  # last time drawn (from time.time())
+
+vd.option('disp_splitwin_pct', 0, 'height of second sheet on screen')
+vd.option('mouse_interval', 1, 'max time between press/release for click (ms)', sheettype=None)
+vd.option('color_sidebar', 'black on 114 blue', 'color of sidebar')
 
-option('disp_splitwin_pct', 0, 'height of second sheet on screen')
-option('mouse_interval', 1, 'max time between press/release for click (ms)', sheettype=None)
 
 class ReturnValue(BaseException):
     'raise ReturnValue(ret) to exit from an inner runresult() with its result.'
@@ -22,7 +28,7 @@ class ReturnValue(BaseException):
 
 @VisiData.api
 def draw_sheet(self, scr, sheet):
-    'Erase *scr* and draw *sheet* on it, including status bars.'
+    'Erase *scr* and draw *sheet* on it, including status bars and sidebar.'
 
     sheet.ensureLoaded()
 
@@ -39,27 +45,72 @@ def draw_sheet(self, scr, sheet):
     except Exception as e:
         self.exceptionCaught(e)
 
-    scr.refresh()
+    try:
+        sidebar = sheet.sidebar
+        sidebar_title = sheet.sidebar_title
+    except Exception as e:
+        vd.exceptionCaught(e)
+        sidebar = str(e)
+        sidebar_title = 'error'
+
+    vd.drawSidebar(scr, sidebar, title=sidebar_title)
 
 
-vd.windowConfig = None
+def iterwraplines(lines, width=80):
+    import textwrap
+    for line in lines:
+        yield from textwrap.wrap(line, width=width, subsequent_indent='  ')
+
+
+@VisiData.api
+def drawSidebar(vd, scr, text, title='sidebar'):
+    if not text:
+        return
+
+    h, w = scr.getmaxyx()
+    maxh, maxw = 0, 0
+
+    lines = list(iterwraplines(text.splitlines(), width=w//2-2))
+
+    maxh = min(h-2, len(lines)+2)
+    maxw = min(w//2, max(map(len, lines))+4)
+
+    sidebar_scr = scr.derwin(maxh, maxw, h-maxh-1, w-maxw-1)
+    clipbox(sidebar_scr, lines, colors.color_sidebar, title=title)
+
+
+vd.windowConfig = dict(pct=0, n=0, h=0, w=0)  # n=top line of bottom window; h=height of bottom window; w=width of screen
 vd.winTop = mock.MagicMock(__bool__=mock.Mock(return_value=False))
+vd.scrMenu = mock.MagicMock(__bool__=mock.Mock(return_value=False))
 
 @VisiData.api
 def setWindows(vd, scr, pct=None):
     'Assign winTop, winBottom, win1 and win2 according to options.disp_splitwin_pct.'
     if pct is None:
         pct = options.disp_splitwin_pct  # percent of window for secondary sheet (negative means bottom)
+    disp_menu = vd.menuRunning or vd.options.disp_menu
+    topmenulines = 1 if disp_menu else 0
     h, w = scr.getmaxyx()
-    n = abs(pct)*h//100
-    # on 100 line screen, pct = 25 means second window on lines 75-100.  pct -25 -> lines 0-25
 
-    desiredConfig = dict(pct=pct, n=n, h=h, w=w)
+    n = 0
+    if pct:
+        # on 100 line screen, pct = 25 means second window on lines 75-100.  pct -25 -> lines 0-25
+        n = abs(pct)*h//100
+        n = min(n, h-topmenulines-3)
+        n = max(3, n)
+
+    desiredConfig = dict(pct=pct, n=n, h=h-topmenulines, w=w)
 
     if vd.scrFull is not scr or vd.windowConfig != desiredConfig:
-        vd.winTop = curses.newwin(n, w, 0, 0)
+        if not topmenulines:
+            vd.scrMenu = None
+        elif not vd.scrMenu:
+            vd.scrMenu = scr.derwin(h, w, 0, 0)
+            vd.scrMenu.keypad(1)
+
+        vd.winTop = scr.derwin(n, w, topmenulines, 0)
         vd.winTop.keypad(1)
-        vd.winBottom = curses.newwin(h-n, w, n, 0)
+        vd.winBottom = scr.derwin(h-n-topmenulines, w, n+topmenulines, 0)
         vd.winBottom.keypad(1)
         if pct == 0 or pct >= 100:  # no second pane
             vd.win1 = vd.winBottom
@@ -79,9 +130,12 @@ def setWindows(vd, scr, pct=None):
         vd.scrFull = scr
         return True
 
+
 @VisiData.api
 def draw_all(vd):
     'Draw all sheets in all windows.'
+    vd.clearCaches()
+
     ss1 = vd.sheetstack(1)
     ss2 = vd.sheetstack(2)
     if ss1 and not ss2:
@@ -90,14 +144,12 @@ def draw_all(vd):
         vd.draw_sheet(vd.win1, ss1[0])
         if vd.win2:
             vd.win2.erase()
-            vd.win2.refresh()
     elif not ss1 and ss2:
         vd.activePane = 2
         vd.setWindows(vd.scrFull)
         vd.draw_sheet(vd.win2, ss2[0])
         if vd.win1:
             vd.win1.erase()
-            vd.win1.refresh()
     elif ss1 and ss2 and vd.win2:
         vd.draw_sheet(vd.win1, ss1[0])
         vd.draw_sheet(vd.win2, ss2[0])
@@ -105,6 +157,17 @@ def draw_all(vd):
         vd.draw_sheet(vd.win1, vd.sheetstack(vd.activePane)[0])
         vd.setWindows(vd.scrFull)
 
+    if vd.scrMenu:
+        vd.drawMenu(vd.scrMenu, vd.activeSheet)
+
+    if vd.win1:
+        vd.win1.refresh()
+    if vd.win2:
+        vd.win2.refresh()
+    if vd.scrMenu:
+        vd.scrMenu.refresh()
+
+
 @VisiData.api
 def runresult(vd):
     try:
@@ -115,6 +178,39 @@ def runresult(vd):
         return e.args[0]
 
 
+@VisiData.api
+def parseMouse(vd, **kwargs):
+    devid, x, y, z, bstate = curses.getmouse()
+
+    found = False
+    for winname, winscr in kwargs.items():
+        py, px = winscr.getparyx()
+        mh, mw = winscr.getmaxyx()
+        if py <= y < py+mh and px <= x < px+mw:
+            y, x, = y-py, x-px
+            found = True
+            # vd.status('clicked at (%s, %s) in %s' % (y, x, winname))
+            break
+
+    clicktype = ''
+    if bstate & curses.BUTTON_CTRL:
+        clicktype += "CTRL-"
+        bstate &= ~curses.BUTTON_CTRL
+    if bstate & curses.BUTTON_ALT:
+        clicktype += "ALT-"
+        bstate &= ~curses.BUTTON_ALT
+    if bstate & curses.BUTTON_SHIFT:
+        clicktype += "SHIFT-"
+        bstate &= ~curses.BUTTON_SHIFT
+
+    keystroke = clicktype + curses.mouseEvents.get(bstate, str(bstate))
+
+    if found:
+        return keystroke, y, x, winname, winscr
+
+    return keystroke, y, x, "whatwin", None
+
+
 @VisiData.api
 def mainloop(self, scr):
     'Manage execution of keystrokes and subsequent redrawing of screen.'
@@ -134,14 +230,16 @@ def mainloop(self, scr):
         sheet = self.activeSheet
 
         if not sheet:
-            continue
+            continue  # waiting for replay to push sheet
 
         threading.current_thread().sheet = sheet
         vd.drawThread = threading.current_thread()
 
-        vd.setWindows(scr)
+        vd.setWindows(vd.scrFull)
 
-        self.draw_all()
+        if not self.drainPendingKeys(scr) or time.time() - self._lastDrawTime > self.min_draw_ms/1000:  #1459
+            self.draw_all()
+            self._lastDrawTime = time.time()
 
         if vd._nextCommands:
             sheet.execCommand(vd._nextCommands.pop(0), keystrokes=self.keystrokes)
@@ -149,7 +247,7 @@ def mainloop(self, scr):
 
         keystroke = self.getkeystroke(scr, sheet)
 
-        if not keystroke and prefixWaiting and ESC in self.keystrokes:  # timeout ESC
+        if not keystroke and prefixWaiting and "Alt+" in self.keystrokes:  # timeout ESC
             self.keystrokes = ''
 
         if keystroke:  # wait until next keystroke to clear statuses and previous keystrokes
@@ -161,32 +259,19 @@ def mainloop(self, scr):
 
             if keystroke == 'KEY_MOUSE':
                 self.keystrokes = ''
-                clicktype = ''
+                keystroke, y, x, winname, winscr = vd.parseMouse(top=vd.winTop, bot=vd.winBottom, menu=vd.scrMenu)
+
+                pct = vd.windowConfig['pct']
+                topPaneActive = ((vd.activePane == 2 and pct < 0)  or (vd.activePane == 1 and pct > 0))
+                bottomPaneActive = ((vd.activePane == 1 and pct < 0)  or (vd.activePane == 2 and pct > 0))
+
+                if (bottomPaneActive and winname == 'top') or (topPaneActive and winname == 'bot'):
+                    self.activePane = 1 if self.activePane == 2 else 2
+                    sheet = self.activeSheet
+
                 try:
-                    devid, x, y, z, bstate = curses.getmouse()
-                    pct = vd.windowConfig['pct']
-                    topPaneActive = ((vd.activePane == 1 and pct < 0)  or (vd.activePane == 2 and pct > 0))
-                    bottomPaneActive = ((vd.activePane == 2 and pct < 0)  or (vd.activePane == 1 and pct > 0))
-                    if vd.windowConfig and y > vd.windowConfig['n'] and topPaneActive:
-                        y -= vd.windowConfig['n']
-                        sheet.mouseX, sheet.mouseY = x, y
-                    elif vd.windowConfig and y < vd.windowConfig['n'] and bottomPaneActive:
-                        sheet.mouseX, sheet.mouseY = x, y
-                    elif pct == 0:
-                        sheet.mouseX, sheet.mouseY = x, y
-                    if bstate & curses.BUTTON_CTRL:
-                        clicktype += "CTRL-"
-                        bstate &= ~curses.BUTTON_CTRL
-                    if bstate & curses.BUTTON_ALT:
-                        clicktype += "ALT-"
-                        bstate &= ~curses.BUTTON_ALT
-                    if bstate & curses.BUTTON_SHIFT:
-                        clicktype += "SHIFT-"
-                        bstate &= ~curses.BUTTON_SHIFT
-
-                    keystroke = clicktype + curses.mouseEvents.get(bstate, str(bstate))
-
-                    f = self.getMouse(scr, x, y, keystroke)
+                    f = self.getMouse(winscr, x, y, keystroke)
+                    sheet.mouseX, sheet.mouseY = x, y
                     if f:
                         if isinstance(f, str):
                             for cmd in f.split():
@@ -194,18 +279,18 @@ def mainloop(self, scr):
                         else:
                             f(y, x, keystroke)
 
-                        self.keystrokes = keystroke
-                        keystroke = ''
+                        self.keystrokes = self.prettykeys(keystroke)
+                        keystroke = ''   # already handled
                 except curses.error:
                     pass
                 except Exception as e:
                     self.exceptionCaught(e)
 
-            if keystroke in self.keystrokes[:-1]:
-                vd.warning('duplicate prefix')
+            if keystroke and keystroke in self.keystrokes[:-1]:
+                vd.warning('duplicate prefix: ' + keystroke)
                 self.keystrokes = ''
             else:
-                self.keystrokes += keystroke
+                self.keystrokes += self.prettykeys(keystroke)
 
         self.drawRightStatus(sheet._scr, sheet)  # visible for commands that wait for input
 
@@ -249,11 +334,11 @@ def initCurses():
     colors.setup()
 
     curses.noecho()
-    curses.cbreak()
 
     curses.raw()    # get control keys instead of signals
     curses.meta(1)  # allow "8-bit chars"
-    curses.mousemask(-1 if options.mouse_interval else 0)
+    curses.MOUSE_ALL = 0xffffffff
+    curses.mousemask(curses.MOUSE_ALL if options.mouse_interval else 0)
     curses.mouseinterval(options.mouse_interval)
     curses.mouseEvents = {}
 
@@ -276,20 +361,25 @@ def wrapper(f, *args, **kwargs):
         curses.endwin()
 
 
-def run(*sheetlist):
+@VisiData.global_api
+def run(vd, *sheetlist):
     'Main entry point; launches vdtui with the given sheets already pushed (last one is visible)'
 
+    scr = None
     try:
         # Populate VisiData object with sheets from a given list.
         for vs in sheetlist:
             vd.push(vs)
 
-        vd.status('Ctrl+H opens help')
-
         scr = initCurses()
         ret = vd.mainloop(scr)
+    except curses.error as e:
+        vd.fail(str(e))
     finally:
-        curses.endwin()
+        if scr:
+            curses.endwin()
+
+    vd.cancelThread(*[t for t in vd.unfinishedThreads if not t.name.startswith('save_')])
 
     if ret:
-        print(ret)
+        vd.printout(ret)
